#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include        <iostream>
#include        <iomanip>
#include        <fstream>
#include	<vector>
#include	<string>
#include	<list>
#include        <stdlib.h>
#include        <math.h>

#include        <glib.h>
#include	<gtk/gtk.h>
#include        <gdk/gdk.h>
#include        <gdk/gdkkeysyms.h>
#include        <gtk/gtkwidget.h>
#include        <gtkgl/gtkglarea.h>
#include	<glade/glade.h>

#include        <GL/gl.h>
#include        <GL/glu.h>

using namespace std;

#include	"mgr.h"
#include	"callbacks.h"


void
on_obj_open_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget*	w = gtk_file_selection_new("Load OBJ");
	Mgr*		mgr = (Mgr *)gtk_object_get_data(GTK_OBJECT(menuitem),"mgr");

  if(!mgr) g_error("on_obj_open_activate: Could not get object manager\n");

  gtk_object_set_data(GTK_OBJECT(w),"mgr",mgr);

  gtk_window_set_position(GTK_WINDOW(w),GTK_WIN_POS_MOUSE);

  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(w)->ok_button),
                     "clicked",
                     GTK_SIGNAL_FUNC(on_obj_ok_button_clicked),
                     w);

  gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(w)->cancel_button),
                     "clicked",
                     GTK_SIGNAL_FUNC(gtk_widget_destroy),
                     GTK_OBJECT(w));

  gtk_widget_show(w);

}


gint on_obj_ok_button_clicked(GtkWidget* widget, GtkWidget *w)
{
	string	filename;
	Mgr*	mgr = (Mgr *)gtk_object_get_data(GTK_OBJECT(w),"mgr");

  if(!mgr) g_error("on_obj_ok_button_clicked: Could not get object manager\n");

  // copy construct filename string
  filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(w));

  // get rid of file selection widget
  // note: destruction of widget also destroys filename string - make sure
  // you copy filename before destroying widget
  gtk_widget_destroy(w);

  // open input stream for reading; ifs will be NULL if file can't be opened
  ifstream ifs(filename.c_str());
  if(!ifs)
    cerr << "Can't open input file: " << filename << endl;
  else {
/* read in object
    if(mgr->obj)
      mgr->obj->clear();
    else
      mgr->obj = new ObjList();
    ifs >> mgr->obj;
*/
    ifs.close();
  }

  return(TRUE);
}


void
on_obj_clear_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	Mgr*	mgr = (Mgr *)gtk_object_get_data(GTK_OBJECT(menuitem),"mgr");

  if(!mgr) g_error("on_obj_clear_activate: Could not get object manager\n");

/* clear object
  if(mgr->obj) {
    mgr->obj->clear();
  }
*/
}


void
on_quit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_main_quit();
}


void
on_display_wire_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
        Mgr*            mgr = (Mgr *)gtk_object_get_data(GTK_OBJECT(menuitem),"mgr");

  // routine to toggle display
  mgr->displayType = DISP_WIRE;

}


void
on_display_flat_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
        Mgr*            mgr = (Mgr *)gtk_object_get_data(GTK_OBJECT(menuitem),"mgr");

  // routine to toggle display
  mgr->displayType = DISP_FLAT;

}


void
on_display_smooth_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
        Mgr*            mgr = (Mgr *)gtk_object_get_data(GTK_OBJECT(menuitem),"mgr");

  // routine to toggle display
  mgr->displayType = DISP_SMTH;

}


void
on_glarea_realize                      (GtkWidget       *widget,
                                        gpointer         user_data)
{

  // OpenGL rendering possible only if we can make gtkGlArea current
  g_return_if_fail(gtk_gl_area_make_current(GTK_GL_AREA(widget)));

  // specify clear color; (0.3,0.4,0.6,1.0) is a nice "slate blue"
  // glClearColor(0.0,0.0,0.0,0.0);
  glClearColor(0.3,0.4,0.6,1.0);

  // turn on blending (for transparency)
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

  // set depth test (z-buffering)
  glClearDepth(1.0);
  glEnable(GL_DEPTH_TEST);

}


gboolean
on_glarea_expose_event                 (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
	Mgr*	mgr = (Mgr *)gtk_object_get_data(GTK_OBJECT(widget),"mgr");

  if(!mgr) g_error("on_glarea_expose_event: Could not get object manager\n");

  // OpenGL rendering possible only if we can make glarea current
  if(!gtk_gl_area_make_current(GTK_GL_AREA(widget)))
    g_error("Couldn't make glarea current");

  // needed?
  glViewport(0,0,widget->allocation.width,widget->allocation.height);

  // view
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0.0,widget->allocation.width,0.0,widget->allocation.height);

  glDrawBuffer(GL_FRONT_AND_BACK);
  glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

  // object
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glDrawBuffer(GL_BACK);

  // draw

  // swap backbuffer to front
  gtk_gl_area_swapbuffers(GTK_GL_AREA(widget));

  glFlush();

  return(TRUE);
}


GtkWidget*
on_glarea_create (gchar *widget_name, gchar *string1, gchar *string2,
                gint int1, gint int2)
{
	GtkWidget	*w=NULL;
	int		attrlist[] =	{
					GDK_GL_RGBA,
					GDK_GL_DOUBLEBUFFER,
					GDK_GL_DEPTH_SIZE, 1,
					GDK_GL_NONE
					};

  //create glarea
  w = GTK_WIDGET(gtk_gl_area_new(attrlist));
  g_assert(w != NULL);

  return(w);
}

gboolean
on_glarea_event_button_press_event     (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  // stub for button pressed over glarea (actually widget below glarea)

  return FALSE;
}


gboolean
on_glarea_event_button_release_event   (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  // stub for button released over glarea (actually widget below glarea)

  return FALSE;
}


gboolean
on_glarea_event_motion_notify_event    (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
  // stub for button motion over glarea (actually widget below glarea)

  return FALSE;
}
